import Axios, { Method, ResponseType, AxiosResponse } from 'axios'

import { message } from 'ant-design-vue';
import 'ant-design-vue/es/message/style/css'; //vite只能用 ant-design-vue/es 而非 ant-design-vue/lib

interface axiosData {
  url: string
  method: Method
  headers?: any
  json: boolean
  contentType?: string
  data?: any
  params?: any
  responseType?: ResponseType
}

let apiBaseUrl = (window as any)._CONFIG['domianURL'] || '/'
const axios = Axios.create({
  baseURL: apiBaseUrl,
  timeout: 3000
})

// 允许携带cookie
axios.defaults.withCredentials = true

// 默认使用 application/json 形式
axios.defaults.headers.post['Content-Type'] = 'application/json'

// 请求拦截器
axios.interceptors.request.use(
  (config) => {
    if (sessionStorage.getItem("accessToken")) {
      config.headers!.Authorization = `${sessionStorage.getItem("accessToken")}`;  
    }
    return config;
  },
  (err) => Promise.reject(err)
)
  
// 响应拦截器
axios.interceptors.response.use(
  (res) => res,
  (err) => {
    if (err.response && err.response.data) {
      const code = err.response.status
      const msg = err.response.data.message
      message.error(`Code: ${code}, Message: ${msg}`)
    } else {
        message.error(`${err}`)
    }
    return Promise.reject(err)
  }
)

export default function request(arr: axiosData) {
  return new Promise<any>((resolve, reject) => {
    axios({
      url: arr.url,
      method: arr.method || 'POST',
      headers: {
        'content-type': arr.contentType ? arr.contentType : arr.json ? 'application/json; charset=UTF-8' : 'application/x-www-form-urlencoded; charset=UTF-8'
      },
      params: arr.params || '',
      data: arr.data || '',
      responseType: arr.responseType || 'json'
    })
      .then((response: AxiosResponse<any>) => {
        /**
         * response格式
         *
         * {
          data:{},
          status:200,
          statusText:'OK',//从服务器返回的http状态文本
          headers: {},//响应头信息
          config: {} //`config`是在请求的时候的一些配置信息
        }
         */
        const responseStatus = `${response.status}`
        // 状态码为2开头的
        if (responseStatus.charAt(0) === '2') {
          if (response.data.code !== '99200') {
            message.error(response.data.message)
            reject(response.data)
            return
          }
          resolve(response.data)
        } else {
          message.error(response.data.message)
          reject(response.data)
        }
      })
        .catch((err) => {
          message.error(err.message)
          reject(err)
        })
  })
}